/**
 * Copyright (C) 2000-2002 the KGhostView authors. See file AUTHORS.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <qfile.h>
#include <qfileinfo.h>
#include <q3frame.h>
#include <q3table.h>
#include <qlayout.h>
#include <qregexp.h>
//Added by qt3to4:
#include <QVBoxLayout>
#include <QList>
#include <QHBoxLayout>

#include <kaction.h>
#include <kactioncollection.h>
#include <kapplication.h>
#include <kconfig.h>
#include <kdebug.h>
#include <kdirwatch.h>
#include <kglobalsettings.h>
#include <kiconloader.h>
#include <kicon.h>
#include <kcomponentdata.h>
#include <kconfiggroup.h>
#include <klocale.h>
#include <kmenu.h>
#include <kstandardaction.h>
#include <kstandardshortcut.h>
#include <ktemporaryfile.h>
#include <kio/scheduler.h>
#include <kaboutdata.h>
#include <ktoggleaction.h>
#include <kselectaction.h>

#include "kgv_view.h"
#include "kgv_miniwidget.h"
#include "kgvconfigdialog.h"
#include "kgvdocument.h"
#include "kgvpagedecorator.h"
#include "kgvpageview.h"
#include "kgvmainwidget.h"
#include "kpswidget.h"
#include "kgvfactory.h"
#include "logwindow.h"
#include "marklist.h"
#include "scrollbox.h"
#include "version.h"
#include "configuration.h"

#include <cmath>

namespace KGV {
    /*
     * This is because Qt's iterators
     * are not standard iterators bc of missing typedefs,
     * so they are only *almost* STL compatible
     */
    template <typename T>
    unsigned distance( T a, T b ) {
	unsigned res = 0;
	while ( a != b ) {
	    ++res;
	    ++a;
	}
	return res;
    }
}

KGVPart::KGVPart( QWidget* parentWidget,
                  QObject* parent,
                  const QStringList &args ) :
    KParts::ReadOnlyPart( parent ),
    _fitTimer( new QTimer( this ) ),
    _job( 0 ),
    _mimetypeScanner( 0 ),
    _dirtyHandler( new QTimer( this ) ),
    _isGuiInitialized( false ),
    _isFileDirty( false ),
    _stickyOptions( false ),
    _embeddedInKGhostView( !args.contains( "KParts::ReadOnlyPart" ) ),
    _customZoomIndex( -1 )
{
    setComponentData( KGVFactory::componentData() );

    // Don't show the progress info dialog if we're embedded in Konqueror.
    setProgressInfoEnabled( !args.contains( "Browser/View") );

    _document = new KGVDocument( this );
    connect( _document, SIGNAL( fileChangeFailed() ),
	     this, SLOT( slotCancelWatch() ) );
    connect( _document, SIGNAL( completed() ),
	     this, SLOT( slotOpenFileCompleted() ) );
    connect( _document, SIGNAL( canceled( const QString& ) ),
	     this, SIGNAL( canceled( const QString& ) ) );

    _fileWatcher = new KDirWatch( this );
    _dirtyHandler->setSingleShot( true );
    connect( _fileWatcher, SIGNAL( dirty( const QString& ) ),
	     this, SLOT( slotFileDirty( const QString& ) ) );
    connect( _dirtyHandler, SIGNAL( timeout() ),
	     this, SLOT( slotDoFileDirty() ) );

    // Setup main widget
    _mainWidget = new KGVMainWidget( parentWidget );
    _mainWidget->setFocusPolicy( Qt::StrongFocus );
    _mainWidget->installEventFilter( this );
    _mainWidget->setAcceptDrops( true );
    connect( _mainWidget, SIGNAL( spacePressed() ),
             this, SLOT( slotReadDown() ) );
    connect( _mainWidget, SIGNAL( urlDropped( const KUrl& ) ),
	    this, SLOT( openUrl( const KUrl& ) ) );

    QHBoxLayout* hlay = new QHBoxLayout( _mainWidget );
    hlay->setSpacing( 0 );
    hlay->setMargin( 0 );
    QVBoxLayout* vlay = new QVBoxLayout( );
    hlay->addItem( vlay );

    const int PAGELIST_WIDTH = 75;

    _scrollBox = new ScrollBox( _mainWidget , "scrollbox" );
    _scrollBox->setFixedWidth( PAGELIST_WIDTH );
    _scrollBox->setMinimumHeight( PAGELIST_WIDTH );
    vlay->addWidget( _scrollBox );

    _divider = new QFrame( _mainWidget );
    _divider->setObjectName( QLatin1String( "divider" ) );
    _divider->setFrameStyle( QFrame::Panel | QFrame::Raised );
    _divider->setLineWidth( 1 );
    _divider->setMinimumWidth( 3 );
    hlay->addWidget( _divider );

    _pageView = new KGVPageView( _mainWidget, "pageview" );
    _pageView->viewport()->setBackgroundRole( QPalette::Mid );
    hlay->addWidget( _pageView, 1 );
    _mainWidget->setFocusProxy( _pageView );
    setWidget( _mainWidget );

    _pageDecorator = new KGVPageDecorator( _pageView->viewport() );
    _pageDecorator->hide();


    _psWidget = new KPSWidget( _pageDecorator );
    _psWidget->readSettings();
    _pageView->setPage( _pageDecorator );
    connect( _psWidget, SIGNAL( output( char*, int ) ),
             this, SLOT( slotGhostscriptOutput( char*, int ) ) );

    connect( _psWidget, SIGNAL( ghostscriptError( const QString& ) ),
             this, SLOT( slotGhostscriptError( const QString& ) ) );


    _logWindow = new LogWindow( i18n( "Ghostscript Messages" ), _mainWidget, "logwindow" );
    _showLogWindow = false;

    connect( _logWindow, SIGNAL( configureGS() ), SLOT( slotConfigure() ) );

    _docManager = new KGVMiniWidget( this );
    _docManager->setPSWidget( _psWidget );
    _docManager->setDocument( document() );

    _markList = new MarkList( _mainWidget, "marklist", _docManager );
    _markList->setFixedWidth( PAGELIST_WIDTH );
    vlay->addWidget( _markList, 1 );
    connect( _markList, SIGNAL( contextMenuRequested ( int, int, const QPoint& ) ),
             this, SLOT( showPopup( int, int, const QPoint& ) ) );


    connect( _markList, SIGNAL( selected( int ) ),
	     _docManager, SLOT( goToPage( int ) ) );
    connect( _docManager, SIGNAL( newPageShown( int ) ),
	     _markList, SLOT( select( int ) ) );
    connect( _docManager, SIGNAL( setStatusBarText( const QString& ) ),
	     this, SIGNAL( setStatusBarText( const QString& ) ) );
    connect( _scrollBox, SIGNAL( valueChangedRelative( int, int ) ),
	     _pageView, SLOT( scrollBy( int, int ) ) );
    connect( _pageView, SIGNAL( pageSizeChanged( const QSize& ) ),
	     _scrollBox, SLOT( setPageSize( const QSize& ) ) );
    connect( _pageView, SIGNAL( viewSizeChanged( const QSize& ) ),
	     _scrollBox, SLOT( setViewSize( const QSize& ) ) );
    connect( _pageView, SIGNAL( contentsMoving( int, int ) ),
	     _scrollBox, SLOT( setViewPos( int, int ) ) );

    //-- File Menu ----------------------------------------------------------
    KStandardAction::saveAs( document(), SLOT( saveAs() ),
                        actionCollection() );
    QAction *action = actionCollection()->addAction( "info" );
    action->setText( i18n( "Document &Info" ) );
    connect(action, SIGNAL(triggered(bool) ), miniWidget(), SLOT( info() ));

    //-- Edit Menu -----------------------------------------------------
    _popup = new KMenu( _markList );
    _popup->setObjectName( "marklist_menu" );

    QAction *act = actionCollection()->addAction( "mark_current" );
    act->setIcon( KIcon("flag") );
    act->setText( i18n( "Mark Current Page" ) );
    connect(act, SIGNAL(triggered(bool) ), _markList, SLOT( markCurrent() ));
    act->setShortcut(Qt::CTRL+Qt::SHIFT+Qt::Key_M);
    _popup->addAction( act );
    act = actionCollection()->addAction( "mark_all" );
    act->setText( i18n( "Mark &All Pages" ) );
    connect(act, SIGNAL(triggered(bool) ), _markList, SLOT( markAll() ));
    _popup->addAction( act );
    act = actionCollection()->addAction( "mark_even" );
    act->setText( i18n( "Mark &Even Pages" ) );
    connect(act, SIGNAL(triggered(bool) ), _markList, SLOT( markEven() ));
    _popup->addAction( act );
    act = actionCollection()->addAction( "mark_odd" );
    act->setText( i18n( "Mark &Odd Pages" ) );
    connect(act, SIGNAL(triggered(bool) ), _markList, SLOT( markOdd() ));
    _popup->addAction( act );
    act = actionCollection()->addAction( "toggle" );
    act->setText( i18n( "&Toggle Page Marks" ) );
    connect(act, SIGNAL(triggered(bool) ), _markList, SLOT( toggleMarks() ));
    _popup->addAction( act );
    act = actionCollection()->addAction( "remove" );
    act->setText( i18n("&Remove Page Marks") );
    connect(act, SIGNAL(triggered(bool) ), _markList, SLOT( removeMarks() ));
    _popup->addAction( act );

    // TODO -- disable entry if there aren't any page names

    //-- View Menu ----------------------------------------------------------
    _selectOrientation = actionCollection()->add<KSelectAction>( "orientation_menu" );
    _selectOrientation->setText( i18n( "&Orientation" ) );
    _selectMedia = actionCollection()->add<KSelectAction>( "media_menu" );
    _selectMedia->setText( i18n( "Paper &Size" ) );

    _flick = actionCollection()->add<KToggleAction>( "no_flicker" );
    _flick->setText( i18n( "No &Flicker" ) );
    connect(_flick, SIGNAL(triggered(bool) ), SLOT( slotFlicker() ));

    QStringList orientations;
    orientations.append( i18n( "Auto" ) );
    orientations.append( i18n( "Portrait" ) );
    orientations.append( i18n( "Landscape" ) );
    orientations.append( i18n( "Upside Down" ) );
    orientations.append( i18n( "Seascape" ) );
    _selectOrientation->setItems( orientations );

    connect( _selectOrientation, SIGNAL( activated( int ) ),
	     this, SLOT( slotOrientation( int ) ) );
    connect( _selectMedia, SIGNAL( activated( int ) ),
             this, SLOT( slotMedia( int ) ) );

    {
	KShortcut zoomInShort = KStandardShortcut::zoomIn();
	zoomInShort.setAlternate(  Qt::CTRL+Qt::Key_Equal  );
	_zoomIn = actionCollection()->addAction( KStandardAction::ZoomIn, "zoomIn",
                                                 this, SLOT( slotZoomIn() ) );
	_zoomIn->setShortcuts( zoomInShort );
    }
    _zoomOut = actionCollection()->addAction( KStandardAction::ZoomOut, "zoomOut", this,  SLOT( slotZoomOut() ) );
    _zoomTo = actionCollection()->add<KSelectAction>( "zoomTo" );
    _zoomTo->setText( i18n( "Zoom" ) );
    _zoomTo->setIcon( KIcon( "zoom-original" ) );
    connect(  _zoomTo, SIGNAL(  triggered(  const QString & ) ), this, SLOT(  slotZoom( const QString& ) ) );
    _zoomTo->setEditable(  true );
    _zoomTo->clear();
    QList<double> mags = DisplayOptions::normalMagnificationValues();
    QStringList zooms;
    int idx = 0;
    int cur = 0;
    for ( QList<double>::iterator first = mags.begin(), last = mags.end();
	    first != last;
	    ++first ) {
        QString str = QString( "%1%" ).arg( KGlobal::locale()->formatNumber( *first * 100.0, 2 ));
        str.remove( KGlobal::locale()->decimalSymbol() + "00" );
        zooms << str;
	if ( *first == 1.0 ) idx = cur;
	++cur;
    }
    _zoomTo->setItems( zooms );
    _zoomTo->setCurrentItem( idx );

    _fitWidth = actionCollection()->addAction("fit_to_page");
    _fitWidth->setText(i18n( "&Fit to Page Width" ));
    connect(_fitWidth, SIGNAL(triggered(bool) ), SLOT( slotFitToPage() ));
    _fitScreen = actionCollection()->addAction("fit_to_screen");
    _fitScreen->setText(i18n( "&Fit to Screen" ));
    connect(_fitScreen, SIGNAL(triggered(bool) ), SLOT( slotFitToScreen() ));
    _fitScreen->setShortcut(Qt::Key_S);

    _prevPage = actionCollection()->addAction( "prevPage" );
    _prevPage->setText( i18n( "Previous Page" ) );
    connect(_prevPage, SIGNAL(triggered(bool) ), SLOT( slotPrevPage() ));
    _prevPage->setShortcut(Qt::CTRL+Qt::Key_PageUp);
    _prevPage->setWhatsThis( i18n( "Moves to the previous page of the document" ) );

    _nextPage = actionCollection()->addAction( "nextPage" );
    _nextPage->setText( i18n( "Next Page" ) );
    connect(_nextPage, SIGNAL(triggered(bool) ), SLOT( slotNextPage() ));
    _nextPage->setShortcut(Qt::CTRL + Qt::Key_PageDown);
    _nextPage->setWhatsThis( i18n( "Moves to the next page of the document" ) );

    _firstPage = actionCollection()->addAction( KStandardAction::FirstPage, "goToStart",
                                                this, SLOT( slotGotoStart() ) );
    _firstPage->setWhatsThis( i18n( "Moves to the first page of the document" ) );

    _lastPage  = actionCollection()->addAction( KStandardAction::LastPage, "goToEnd", this, SLOT( slotGotoEnd() ) );
    _lastPage->setWhatsThis( i18n( "Moves to the last page of the document" ) );

    KShortcut readUpShort = KStandardShortcut::shortcut( KStandardShortcut::Prior );
    readUpShort.setAlternate( ( Qt::SHIFT+Qt::Key_Space ) );
    _readUp = actionCollection()->addAction( "readUp" );
    _readUp->setIcon( KIcon("go-up") );
    _readUp->setText( i18n( "Read Up" ) );
    connect(_readUp, SIGNAL(triggered(bool) ), SLOT( slotReadUp() ));
    _readUp->setShortcuts(readUpShort);

    KShortcut readDownShort = KStandardShortcut::shortcut( KStandardShortcut::Next );
    readDownShort.setAlternate( ( Qt::Key_Space ) );
    _readDown = actionCollection()->addAction( "readDown" );
    _readDown->setIcon( KIcon("go-down") );
    _readDown->setText( i18n( "Read Down" ) );
    connect(_readDown, SIGNAL(triggered(bool) ), SLOT( slotReadDown() ));
    _readDown->setShortcuts(readDownShort);

    _gotoPage = actionCollection()->addAction( KStandardAction::GotoPage, "goToPage",
                                               _docManager, SLOT( goToPage() ) );

    //-- Settings Menu ------------------------------------------------------
    _showScrollBars = actionCollection()->add<KToggleAction>( "show_scrollbars" );
    _showScrollBars->setText( i18n( "Show &Scrollbars" ) );
    _showScrollBars->setCheckedState(KGuiItem(i18n("Hide &Scrollbars")));
    _watchFile = actionCollection()->add<KToggleAction>( "watch_file" );
    _watchFile->setText( i18n( "&Watch File" ) );
    connect(_watchFile, SIGNAL(triggered(bool) ), SLOT( slotWatchFile() ));
    _showPageList = actionCollection()->add<KToggleAction>( "show_page_list" );
    _showPageList->setText( i18n( "Show &Page List" ) );
    _showPageList->setCheckedState(KGuiItem(i18n("Hide &Page List")));
    _showPageLabels = actionCollection()->add<KToggleAction>( "show_page_labels" );
    _showPageLabels->setText( i18n("Show Page &Labels") );
    _showPageLabels->setCheckedState(KGuiItem(i18n("Hide Page &Labels")));
    KStandardAction::preferences( this, SLOT( slotConfigure() ), actionCollection() );
    connect( _showScrollBars, SIGNAL( toggled( bool ) ),
             SLOT( showScrollBars( bool ) ) );
    connect( _showPageList, SIGNAL( toggled( bool ) ),
             SLOT( showMarkList( bool ) ) );
    connect( _showPageLabels, SIGNAL( toggled( bool ) ),
             SLOT( showPageLabels( bool ) ) );

    _extension = new KGVBrowserExtension( this );

    setXMLFile( "kgv_part.rc" );

    connect( miniWidget(), SIGNAL( newPageShown( int ) ),
	     this, SLOT( slotNewPage( int ) ) );
    connect( _pageView, SIGNAL( contentsMoving( int, int ) ),
	     this, SLOT( slotPageMoved( int, int ) ) );

    connect( _pageView, SIGNAL( nextPage() ), SLOT( slotNextPage() ));
    connect( _pageView, SIGNAL( prevPage() ), SLOT( slotPrevPage() ));
    connect( _pageView, SIGNAL( zoomIn() ), SLOT( slotZoomIn() ));
    connect( _pageView, SIGNAL( zoomOut() ), SLOT( slotZoomOut() ));
    connect( _pageView, SIGNAL( ReadUp() ), SLOT( slotReadUp() ));
    connect( _pageView, SIGNAL( ReadDown() ), SLOT( slotReadDown() ));

    QStringList items = document()->mediaNames();
    items.prepend( i18n( "Auto ") );
    _selectMedia->setItems( items );

    readSettings();

    updatePageDepActions();
}

KGVPart::~KGVPart()
{
    if ( _job ) _job -> kill();
    delete _mimetypeScanner;
    writeSettings();
}

KAboutData* KGVPart::createAboutData()
{
    KAboutData* about = new KAboutData( "kghostview", 0, ki18n( "KGhostView"),
            KGHOSTVIEW_VERSION,
            ki18n( "Viewer for PostScript (.ps, .eps) and Portable Document Format (.pdf) files"),
            KAboutData::License_GPL,
            ki18n("(C) 1998 Mark Donohoe, (C) 1999-2000 David Sweet, "
            "(C) 2000-2003 Wilco Greven"),
            ki18n( "KGhostView displays, prints, and saves "
                       "PostScript and PDF files.\n"
                       "Based on original work by Tim Theisen." ) );
    about->addAuthor( ki18n("Luís Pedro Coelho"),
                      ki18n( "Current maintainer" ),
                      "luis@luispedro.org",
                      "http://luispedro.org" );
    about->addAuthor( ki18n("Wilco Greven"),
                      ki18n( "Maintainer 2000-2003" ),
                      "greven@kde.org" );
    about->addAuthor( ki18n("David Sweet"),
                      ki18n( "Maintainer 1999-2000" ),
                      "dsweet@kde.org",
                      "http://www.andamooka.org/~dsweet" );
    about->addAuthor( ki18n("Mark Donohoe"),
	 	      ki18n( "Original author" ),
                      "donohoe@kde.org" );
    about->addAuthor( ki18n("David Faure"),
		      ki18n( "Basis for shell"),
                      "faure@kde.org" );
    about->addAuthor( ki18n("Daniel Duley"),
		      ki18n( "Port to KParts" ),
                      "mosfet@kde.org" );
    about->addAuthor( ki18n("Espen Sand"),
		      ki18n( "Dialog boxes" ),
                      "espen@kde.org" );
    about->addCredit( ki18n("Russell Lang of Ghostgum Software Pty Ltd"),
		      ki18n( "for contributing GSView's DSC parser." ),
                      0,
                      "http://www.ghostgum.com.au/" );
    about->addCredit( ki18n("The Ghostscript authors"),
                      KLocalizedString(), 0,
		      "http://www.cs.wisc.edu/~ghost/" );
    return about;
}

bool KGVPart::closeUrl()
{
    document()->close();
    _psWidget->stopInterpreter();
    _docManager->getThumbnailService()->reset();
    _markList->clear();
    _pageDecorator->hide();
    _scrollBox->clear();
    _isFileDirty = false;
    if ( _job )
    {
	_job -> kill();
	_job = 0;
    }
    if( _mimetypeScanner != 0 )
	_mimetypeScanner->abort();
    if( !localFilePath().isEmpty() )
        _fileWatcher->removeFile( localFilePath() );
    _mimetype.clear();
    updatePageDepActions();
    stateChanged( "initState" );
    return KParts::ReadOnlyPart::closeUrl();
}

void KGVPart::writeSettings()
{
    KConfigGroup general( KGVFactory::componentData().config(), "General" );
    if ( !_embeddedInKGhostView )
        general.writeEntry( "Display Options", DisplayOptions::toString( miniWidget()->displayOptions() ) );
    general.sync();
}

void KGVPart::readSettings()
{
    KConfigGroup general( KGVFactory::componentData().config(), "General" );

    _showScrollBars->setChecked( Configuration::showScrollBars() );
    showScrollBars( _showScrollBars->isChecked() );

    _watchFile->setChecked( Configuration::watchFile() );
    slotWatchFile();

    _showPageList->setChecked( Configuration::showPageList() );
    showMarkList( _showPageList->isChecked() );

    _showPageLabels->setChecked( Configuration::watchFile() );
    showPageLabels( _showPageLabels->isChecked() );

    _showLogWindow = Configuration::messages();
    if ( !_embeddedInKGhostView ) {
        DisplayOptions options;
        if ( DisplayOptions::fromString( options, general.readEntry( "Display Options" ) ) )
            setDisplayOptions( options );
    }
    _psWidget->readSettings();
}

void KGVPart::slotScrollLeft()
{
    _pageView->scrollLeft();
}

void KGVPart::slotFlicker()
{
    if ( _psWidget ) _psWidget->setDoubleBuffering( _flick->isChecked() );
}

void KGVPart::slotScrollRight()
{
    _pageView->scrollRight();
}

void KGVPart::slotScrollUp()
{
    _pageView->scrollUp();
}

void KGVPart::slotScrollDown()
{
    _pageView->scrollDown();
}

void KGVPart::slotReadUp()
{
    if( !( document() && document()->isOpen() ) )
	return;

    if( !_pageView->readUp() ) {
	if (_docManager->prevPage())
	    _pageView->scrollBottom();
    }
}

void KGVPart::slotReadDown()
{
    if( !( document() && document()->isOpen() ) )
	return;

    if( !_pageView->readDown() ) {
	if( _docManager->nextPage() )
	   _pageView->scrollTop();
    }
}

void KGVPart::slotPrevPage()
{
    if( !document() || !document()->isOpen() ) return;
    _docManager->prevPage();
}

void KGVPart::slotNextPage()
{
    if( !document() || !document()->isOpen() ) return;
    _docManager->nextPage();
}

void KGVPart::slotGotoStart()
{
    _docManager->firstPage();
    _pageView->scrollTop();
}

void KGVPart::slotGotoEnd()
{
    _docManager->lastPage();
    _pageView->scrollTop();
}

void KGVPart::slotWatchFile()
{
    if( _watchFile->isChecked() )
	_fileWatcher->startScan();
    else {
	_dirtyHandler->stop();
	_fileWatcher->stopScan();
    }
}

void KGVPart::slotCancelWatch()
{
    _fileWatcher->stopScan();
    _watchFile->setChecked( false );
}

/*
void KGVPart::slotFitWidth()
{
    _docManager->fitWidth( pageView()->viewport()->width() -
         2*( pageDecorator()->margin() + pageDecorator()->borderWidth() ) );
}
*/

void KGVPart::updateZoomActions()
{
    if( !( document() && document()->isOpen() ) )
	return;

    _zoomIn->setEnabled(!_docManager->atMaxZoom());
    _zoomOut->setEnabled(!_docManager->atMinZoom());
    _zoomTo->setEnabled( true );
    QStringList items = _zoomTo->items();
    bool updateItems = false;
    if (_customZoomIndex != -1)
    {
        items.removeAt(_customZoomIndex);
        _customZoomIndex = -1;
        updateItems = true;
    }
    double zoom = floor(miniWidget()->displayOptions().magnification()*1000.0) / 10.0;
    unsigned idx = 0;
    for ( QStringList::iterator first = items.begin(), last = items.end();
	    first != last;
	    ++first ) {
	QString cur = *first;
	cur.remove(  cur.indexOf(  '%' ), 1 );
	cur = cur.simplified();
	bool ok = false;
	double z = cur.toDouble(&ok);
	if ( ok ) {
            if (std::abs( z - zoom ) < 0.1 ) {
                if (updateItems)
                    _zoomTo->setItems( items );
	        _zoomTo->setCurrentItem( idx );
	        return;
            }
            if ( z > zoom )
                break;
	}
	++idx;
    }

    // Show percentage that isn't predefined
    QString str = QString( "%1%" ).arg( KGlobal::locale()->formatNumber( zoom, 2 ));
    str.remove( KGlobal::locale()->decimalSymbol() + "00" );
    items.insert( idx, str );
    _zoomTo->setItems( items );
    _zoomTo->setCurrentItem( idx );
    _customZoomIndex = idx;
}

void KGVPart::updatePageDepActions()
{
    bool hasDoc = document() && document()->isOpen();

    _fitWidth->setEnabled( hasDoc );
    _fitScreen->setEnabled( hasDoc );

    _prevPage->setEnabled( hasDoc && !_docManager->atFirstPage() );
    _firstPage->setEnabled( hasDoc && !_docManager->atFirstPage() );
    _nextPage->setEnabled( hasDoc && !_docManager->atLastPage() );
    _lastPage->setEnabled( hasDoc && !_docManager->atLastPage() );
    _gotoPage->setEnabled( hasDoc &&
          !(_docManager->atFirstPage() && _docManager->atLastPage()) );

    updateReadUpDownActions();
}

void KGVPart::updateReadUpDownActions()
{
    if( !( document() && document()->isOpen() ) )
    {
	_readUp->setEnabled( false );
	_readDown->setEnabled( false );
	return;
    }

    if( _docManager->atFirstPage() && _pageView->atTop() )
	_readUp->setEnabled( false );
    else
	_readUp->setEnabled( true );

    if( _docManager->atLastPage() && _pageView->atBottom() )
	_readDown->setEnabled( false );
    else
	_readDown->setEnabled( true );
}

bool KGVPart::openUrl( const KUrl& url )
{
    if( !url.isValid() )
	return false;
    if( !closeUrl() )
	return false;

    setUrl( url );
    if ( !_stickyOptions ) _options.reset();

    emit setWindowCaption( this->url().prettyUrl() );

    _mimetypeScanner = new KGVRun( this->url(), 0, this->url().isLocalFile(), false );
    connect( _mimetypeScanner, SIGNAL( finished( const QString& ) ),
             SLOT( slotMimetypeFinished( const QString& ) ) );
    connect( _mimetypeScanner, SIGNAL( error() ),
             SLOT( slotMimetypeError() ) );

    return true;
}

void KGVPart::openUrlContinue()
{
    kDebug(4500) << "KGVPart::openUrlContinue()";
    if( url().isLocalFile() )
    {
	emit started( 0 );
        setLocalFilePath( url().path() );
	document()->openFile( localFilePath(), _mimetype );
    }
    else
    {
        setLocalFileTemporary( true );
	// Use same extension as remote file. This is important for
	// mimetype-determination (e.g. koffice)
	QString extension;
	QString fileName = url().fileName();
	int extensionPos = fileName.lastIndexOf( '.' );
	if( extensionPos != -1 )
	    extension = fileName.mid( extensionPos ); // keep the '.'
	KTemporaryFile tempFile;
	tempFile.setSuffix(extension);
	tempFile.setAutoRemove(false);
	tempFile.open();
        setLocalFilePath( tempFile.fileName() );
	_tmpFile.setFileName( localFilePath() );
	_tmpFile.open( QIODevice::ReadWrite );

	/*
	d->m_job = KIO::file_copy( m_url, m_file, 0600, true, false, d->m_showProgressInfo );
	emit started( d->m_job );
	connect( d->m_job, SIGNAL( result( KJob * ) ), this, SLOT( slotJobFinished ( KJob * ) ) );
	*/

	_job = KIO::get( url(), KIO::NoReload, isProgressInfoEnabled() ? KIO::DefaultFlags : KIO::HideProgressInfo );

	connect( _job, SIGNAL( data( KIO::Job*, const QByteArray& ) ),
		 SLOT( slotData( KIO::Job*, const QByteArray& ) ) );
	connect( _job, SIGNAL( result( KJob* ) ),
		 SLOT( slotJobFinished( KJob* ) ) );

	emit started( _job );
    }
}

bool KGVPart::openFile()
{
    return false;
}

void KGVPart::slotOpenFileCompleted()
{
    _docManager->getThumbnailService()->setEnabled( true );
    if( _isFileDirty )
    {
	_docManager->redisplay();
	_isFileDirty = false;
    }
    else
    {
	if ( !_stickyOptions ) setDisplayOptions( DisplayOptions() );
	_stickyOptions = false;

	stateChanged( "documentState" );
	if ( !_fileWatcher->contains( localFilePath() ) )
	    _fileWatcher->addFile( localFilePath() );
	slotWatchFile();
	updateZoomActions();
	emit completed();
    }
}

void KGVPart::slotGhostscriptOutput( char* data, int len )
{
    _logWindow->append( QString::fromLocal8Bit( data, len ) );
    if( _showLogWindow )
	_logWindow->show();
}


void KGVPart::slotGhostscriptError( const QString& error )
{
    _logWindow->setLabel( i18n( "<qt>An error occurred in rendering.<br>"
				"<strong>%1</strong><br>"
				"The display may contain errors.<br>"
				"Below are any error messages which were received from Ghostscript "
				"(<nobr><strong>%2</strong></nobr>) "
				"which may help you.</qt>" ,
		      error ,
		      Configuration::interpreter() ),
	   true );
    // The true above makes it show a "configure gs" option, but maybe we
    // should trigger an auto-redetection?
    // LPC (13 Apr 2003)
    _logWindow->show();
}


void KGVPart::guiActivateEvent( KParts::GUIActivateEvent* event )
{
    if( event->activated() && !_isGuiInitialized )
    {
	stateChanged( "initState" );
	_isGuiInitialized = true;
    }
    KParts::ReadOnlyPart::guiActivateEvent( event );
}

void KGVPart::slotData( KIO::Job* job, const QByteArray& data )
{
    Q_ASSERT( _job == job );

    kDebug(4500) << "KGVPart::slotData: received " << data.size() << " bytes.";

    _tmpFile.write( data );
}

void KGVPart::slotMimetypeFinished( const QString& type )
{
    kDebug(4500) << "KGVPart::slotMimetypeFinished( " << type << " )";
    _mimetype = type;
    if( !_mimetypeScanner || _mimetypeScanner->hasError() )
	emit canceled( QString::null );	//krazy:exclude=nullstrassign for old broken gcc
    else
	openUrlContinue();
    _mimetypeScanner = 0;
}

void KGVPart::slotMimetypeError()
{
    kDebug(4500) << "KGVPart::slotMimetypeError()";
    _mimetypeScanner = 0;
    emit started( 0 );
    //kapp->processEvents();
    emit canceled( QString::null );	//krazy:exclude=nullstrassign for old broken gcc
}

void KGVPart::slotJobFinished( KJob* job )
{
    Q_ASSERT( _job == job );

    kDebug(4500) << "KGVPart::slotJobFinished";

    _job = 0;

    _tmpFile.close();

    if( job->error() )
	emit canceled( job->errorString() );
    else
	document()->openFile( localFilePath(), _mimetype );
}

void KGVPart::slotFileDirty( const QString& fileName )
{
    // The beauty of this is that each start cancels the previous one.
    // This means that timeout() is only fired when there have
    // no changes to the file for the last 750 milisecs.
    // This is supposed to ensure that we don't update on every other byte
    // that gets written to the file.
    if ( fileName == localFilePath() )
    {
	_dirtyHandler->start( 750 );
    }
}

void KGVPart::slotDoFileDirty()
{
	kDebug(4500) << "KGVPart::File changed";
	_isFileDirty = true;
	reloadFile();
}

void KGVPart::slotNewPage( int )
{
    updatePageDepActions();
    //media->setCurrentItem (miniWidget()->getSize()-1);
    //orientation->setCurrentItem (miniWidget()->getOrientation()-1);
    //TODO -- zoom
}

void KGVPart::slotPageMoved( int, int )
{
    updateReadUpDownActions();
}

void KGVPart::slotOrientation( int id )
{
    switch( id ) {
    case 0: miniWidget()->restoreOverrideOrientation();              break;
    case 1: miniWidget()->setOverrideOrientation( CDSC_PORTRAIT );   break;
    case 2: miniWidget()->setOverrideOrientation( CDSC_LANDSCAPE );  break;
    case 3: miniWidget()->setOverrideOrientation( CDSC_UPSIDEDOWN ); break;
    case 4: miniWidget()->setOverrideOrientation( CDSC_SEASCAPE );   break;
    default: ;
    }
}

void KGVPart::slotMedia( int id )
{
    if( id == 0 )
	miniWidget()->restoreOverridePageMedia();
    else
	miniWidget()->setOverridePageMedia( _document->mediaNames()[id-1] );
}

void KGVPart::showScrollBars( bool show )
{
    _pageView->enableScrollBars( show );
}

void KGVPart::showMarkList( bool show )
{
    _markList->setVisible( show );
    _scrollBox->setVisible( show );
    _divider->setVisible( show );
}

void KGVPart::showPageLabels( bool show )
{
    _docManager->enablePageLabels( show );
}

void KGVPart::slotZoomIn()
{
    _docManager->zoomIn();
    updateZoomActions();
}

void KGVPart::slotZoomOut()
{
    _docManager->zoomOut();
    updateZoomActions();
}

void KGVPart::slotZoom( const QString& nz )
{
    QString z = nz;
    double zoom;
    z.remove(  z.indexOf(  '%' ), 1 );
    zoom = KGlobal::locale()->readNumber(  z ) / 100;
    kDebug( 4500 ) << "ZOOM = "  << nz << ", setting zoom = " << zoom;

    DisplayOptions options = miniWidget()->displayOptions();
    options.setMagnification( zoom );
    miniWidget()->setDisplayOptions( options );
    miniWidget()->redisplay();
    _mainWidget->setFocus();
    updateZoomActions();
}

void KGVPart::slotFitToPage()
{
    kDebug(4500) << "KGVPart::slotFitToPage()";
    if( pageView()->page() )
	miniWidget()->fitWidth( pageView()->viewport()->width() - 16 );
    // We subtract 16 pixels because of the page decoration.
    updateZoomActions();
}

void KGVPart::slotFitToScreen()
{
    kDebug(4500) << "KGVPart::slotFitToScreen()";
    if ( _fitTimer->isActive() ) {
	disconnect( _fitTimer, SIGNAL( timeout() ), this, 0 );
	connect( _fitTimer, SIGNAL( timeout() ), SLOT( slotDoFitToScreen() ) );
    }
    else slotDoFitToScreen();
}

void KGVPart::slotDoFitToScreen()
{
    kDebug(4500) << "KGVPart::slotDoFitToScreen()";
    if( pageView()->page() )
	miniWidget()->fitWidthHeight( pageView()->viewport()->width() - 16,
					pageView()->viewport()->height() - 16 );
    updateZoomActions();
}

void KGVPart::reloadFile()
{
    _psWidget->stopInterpreter();
    _docManager->getThumbnailService()->reset();
    document()->openFile( localFilePath(), _mimetype );
}

void KGVPart::slotConfigure()
{
    ConfigDialog::showSettings( this );
}

void KGVPart::slotConfigurationChanged()
{
    readSettings();
    _psWidget->readSettings();
    miniWidget()->redisplay();
}

void KGVPart::setDisplayOptions( const DisplayOptions& options )
{
    kDebug(4500) << "KGVPart::setDisplayOptions()";
    _stickyOptions = true;
    _markList->select( options.page() );
    _docManager->setDisplayOptions( options );
    _selectOrientation->setCurrentItem( options.overrideOrientation()  );
    QStringList medias = document()->mediaNames();
    int ind = medias.indexOf( options.overridePageMedia() );
    if ( ind >= 0 ){
	// The options are displayed in inverted order.
	// Therefore, size() - index gets you the display index
	_selectMedia->setCurrentItem(  medias.size() - KGV::distance( medias.begin(), medias.begin() + ind ) );
    } else {
	_selectMedia->setCurrentItem( 0 );
    }
}


KGVBrowserExtension::KGVBrowserExtension( KGVPart *parent ) :
    KParts::BrowserExtension( parent )
{
    setObjectName("KGVBrowserExtension");
    emit enableAction( "print", true );
    setURLDropHandlingEnabled( true );
}

void KGVBrowserExtension::print()
{
    ((KGVPart *)parent())->document()->print();
}


KGVRun::KGVRun( const KUrl& url, mode_t mode, bool isLocalFile,
                bool showProgressInfo ) :
    KRun( url,0L, mode, isLocalFile, showProgressInfo )
{
    connect( this, SIGNAL( finished() ), SLOT( emitFinishedWithMimetype() ) );
}

KGVRun::~KGVRun()
{}

void KGVRun::foundMimeType( const QString& mimetype )
{
    kDebug(4500) << "KGVRun::foundMimeType( " << mimetype << " )";

    KIO::TransferJob *job = ::qobject_cast< KIO::TransferJob* >( this->job() );
    if( job )
    {
	job->putOnHold();
	setJob(0);
    }

    _mimetype = mimetype;

    setFinished( true );
    timer().start( 0 );
}


void KGVPart::updateFullScreen( bool fs )
{
    if ( fs ) showMarkList( false );
    else showMarkList( _showPageList->isChecked() );
}

void KGVPart::showPopup( int, int, const QPoint& pos )
{
    _popup->exec( pos );
}

#include "kgv_view.moc"


// vim:sw=4:sts=4:ts=8:sta:tw=78:noet
